package member_center

import (
	"applet/app/db"
	"applet/app/e"
	md2 "applet/app/md/member_center"
	"applet/app/utils"
	"code.fnuoos.com/EggPlanet/egg_models.git/src/implement"
	"code.fnuoos.com/EggPlanet/egg_models.git/src/model"
	"github.com/gin-gonic/gin"
	"time"
)

// GetTagList
// @Summary      制度中心-会员中心-标签管理(获取)
// @Tags         会员中心
// @Description  标签管理(获取)
// @Accept       json
// @Produce      json
// @param Authorization header string true "验证参数Bearer和token空格拼接"
// @Param    limit    query	 int 	true  "每页大小"
// @Param    page    query	 int 	true  "页数"
// @Success      200   {object}  md.GetTagListResp    	"具体数据"
// @Failure      400   {object}   md.Response              	"具体错误"
// @Router       /api/memberCenter/tagManagement/getTagList [get]
func GetTagList(c *gin.Context) {
	page := c.DefaultQuery("page", "1")
	limit := c.DefaultQuery("limit", "10")

	sql := "SElECT count(distinct uid) as total, tag_id FROM `user_tag_records` GROUP BY tag_id"
	results, err := db.QueryNativeString(db.Db, sql)
	if err != nil {
		e.OutErr(c, e.ERR_DB_ORM, err.Error())
		return
	}
	var countMap = map[int64]string{}
	for _, res := range results {
		countMap[utils.StrToInt64(res["tag_id"])] = res["total"]
	}

	tagDb := implement.NewUserTagDb(db.Db)
	tags, total, err := tagDb.UserTagAllByAscByPage(utils.StrToInt(page), utils.StrToInt(limit))
	if err != nil {
		e.OutErr(c, e.ERR_DB_ORM, err.Error())
		return
	}

	list := make([]md2.TagListNode, len(*tags))
	for i, tag := range *tags {
		list[i].Name = tag.TagName
		list[i].Count = countMap[int64(tag.Id)]
		list[i].CreateAt = tag.CreateAt
		list[i].Memo = tag.Memo
		list[i].TagID = utils.IntToStr(tag.Id)
		list[i].IsPunish = tag.IsPunish
	}

	resp := md2.GetTagListResp{
		List: list,
		Paginate: md2.Paginate{
			Limit: utils.StrToInt(limit),
			Page:  utils.StrToInt(page),
			Total: total,
		},
	}
	e.OutSuc(c, resp, nil)
}

// AddTag
// @Summary      制度中心-会员中心-标签管理(新增)
// @Tags         会员中心
// @Description  标签管理(新增)
// @Accept       json
// @Produce      json
// @param Authorization header string true "验证参数Bearer和token空格拼接"
// @Param    req    body	 md.AddTagReq 	true  "新增的标签信息"
// @Success      200   {int}     	"新增的标签 ID"
// @Failure      400   {object}   md.Response              	"具体错误"
// @Router       /api/memberCenter/tagManagement/addTag [post]
func AddTag(c *gin.Context) {
	var req *md2.AddTagReq
	if err := c.ShouldBindJSON(&req); err != nil {
		e.OutErr(c, e.ERR_INVALID_ARGS, err.Error())
		return
	}

	tag := model.UserTag{
		TagName:  req.Name,
		Memo:     req.Memo,
		CreateAt: time.Now().Format("2006-01-02 15:04:05"),
	}
	if req.IsPunish == "1" {
		tag.IsPunish = 1
	}
	tagDb := implement.NewUserTagDb(db.Db)
	tagID, err := tagDb.UserTagInsert(&tag)
	if err != nil {
		e.OutErr(c, e.ERR_DB_ORM, err.Error())
		return
	}
	e.OutSuc(c, tagID, nil)
}

// UpdateTag
// @Summary      制度中心-会员中心-标签管理(更新)
// @Tags         会员中心
// @Description  标签管理(更新)
// @Accept       json
// @Produce      json
// @param Authorization header string true "验证参数Bearer和token空格拼接"
// @Param    req    body	 md.UpdateTagReq 	true  "需要修改的标签信息"
// @Success      200   {int}     	"成功修改数据条数"
// @Failure      400   {object}   md.Response              	"具体错误"
// @Router       /api/memberCenter/tagManagement/updateTag [post]
func UpdateTag(c *gin.Context) {
	var req *md2.UpdateTagReq
	if err := c.ShouldBindJSON(&req); err != nil {
		e.OutErr(c, e.ERR_INVALID_ARGS, err.Error())
		return
	}
	tagID := utils.StrToInt(req.TagID)

	tag := &model.UserTag{
		Id:      tagID,
		TagName: req.Name,
		Memo:    req.Memo,
	}

	forceColumns := []string{"tag_name", "memo", "is_punish"}

	tagDb := implement.NewUserTagDb(db.Db)
	affected, err := tagDb.UserTagUpdate(tagID, tag, forceColumns...)
	if err != nil {
		e.OutErr(c, e.ERR_DB_ORM, err.Error())
		return
	}
	e.OutSuc(c, affected, nil)
}

// DeleteTag
// @Summary      制度中心-会员中心-标签管理(删除)
// @Tags         会员中心
// @Description  标签管理(删除)
// @Accept       json
// @Produce      json
// @param Authorization header string true "验证参数Bearer和token空格拼接"
// @Param    req    body	 md.DeleteTagReq 	true  "需要删除的标签 ID"
// @Success      200   {int}     	"成功删除标签用户数"
// @Failure      400   {object}   md.Response              	"具体错误"
// @Router       /api/memberCenter/tagManagement/deleteTag [delete]
func DeleteTag(c *gin.Context) {
	var req *md2.DeleteTagReq
	if err := c.ShouldBindJSON(&req); err != nil {
		e.OutErr(c, e.ERR_INVALID_ARGS, err.Error())
		return
	}

	tagDb := implement.NewUserTagDb(db.Db)
	recordsDb := implement.NewUserTagRecordsDb(db.Db)
	session := db.Db.NewSession()
	defer session.Close()

	_, err := tagDb.UserTagDeleteBySession(session, req.TagID)
	if err != nil {
		session.Rollback()
		e.OutErr(c, e.ERR_DB_ORM, err.Error())
		return
	}
	affected, err := recordsDb.UserTagRecordsDeleteBySession(session, map[string]interface{}{
		"key":   "id",
		"value": req.TagID,
	})
	if err != nil {
		session.Rollback()
		e.OutErr(c, e.ERR_DB_ORM, err.Error())
		return
	}
	e.OutSuc(c, affected, nil)
}
